/*
*  This code simulates the dynamics of the susceptible-infected-refractory-susceptible (SIRS) model with a bimodal distribution of integrators.
*  It can be compiled with the following command: 
*  gcc SIRSdiversity.c -lm
* 
* The output files are :
* 
* 1- Raster plot (rasterBimodald_psynR0.055000.dat)
*  in which the first column represents the time index of a spike,
*  and the second column the node index.
* 
* 2- Instantaneous firing rate (rateBimodald_psynR0.055000.dat) 
* in which the 1st column represents the time, 
* 2nd column, instantaneous firing rate of the whole network,
* 3rd column, instantaneous firing rate of integrators,
* 4th column, instantaneous firing rate of nonintegrators. 
* 
* Reference: 
* "Diversity improves performance in excitable networks"
* 2016
* by: Leonardo L. Gollo, Mauro Copelli, and James A. Roberts
* 
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<string.h>

#define G 0
#define K -1
#define NPontosr 1
#define NReal  1
#define meansyn 50.0
#define pzinho 0.0
#define plambdamax  0.15
#define psynR 0.055
#define maxtheta 2.0
#define maxthetaint 2
#define iiprop 0.50
#define M 5000

// Functions
void distributing (void);
void iteration (void);
void arquivo0 (void);
int declaring (void);
int connectionsyn (void);
float ran2 (long *idum);


// Variables
long seed;
int ESTij[M], Nvizinhossyn[M], Nvizinhosauxsyn[M], vizinhossyn[M][M + 1];
double Pmedio[G + 1], alpha, beta, lambdazinho, lambda[G + 1], Fa[G + 1],
  psyn;
double Ftotal[NPontosr][NReal], St, sigma0[NPontosr], sigmat[NPontosr], fator,
  F0[NPontosr][NReal], F0m[NPontosr], Ftotalm[NPontosr];
double umporrrange, r0, r, Fauxtheta[maxthetaint][NPontosr], Ftotal0,
  Fauxtheta1, Fauxtheta2;
int i, thetai;
long int k, l, infinity, t, n, jj, ii, mm, xx, kki;
int ij, temp, ifp, thetau[M], auxthetam;
unsigned long int j, jmax, Ni, mi, Nsyn;
int m;
FILE *outfileRaster, *outfileRate;
struct synapse
{
  int ar1;
  int ar2;
};
struct synapse *syn;

/********calculating*******/

void
calculating (void)
{
  int umporDeltapsyn, tmax;

  tmax = 5000;
  umporrrange = 1.e11;

  alpha = 1.0;
  beta = 0.5;

  umporDeltapsyn = 1;

  r0 = 1.e-9;
  fator = pow (umporrrange, (1. / (NPontosr - 1.)));
  arquivo0 ();

  for (kki = 0; kki < umporDeltapsyn; kki++)	//for linear em p
    {
      psyn = psynR;
      {
	n = declaring ();
	for (jj = 0; jj < NPontosr; jj++)
	  {
	    sigma0[jj] = 0.;
	    sigmat[jj] = 0.;
	    for (auxthetam = 0; auxthetam < maxtheta; auxthetam++)
	      {
		Fauxtheta[auxthetam][jj] = 0.0;
	      }
	    F0m[jj] = 0.;
	    Ftotalm[jj] = 0.;
	    for (ii = 0; ii < NReal; ii++)
	      {
		F0[jj][ii] = 0.;
		Ftotal[jj][ii] = 0.;
	      }
	  }

	for (ii = 0; ii <= G; ii++)
	  {
	    Fa[ii] = 0;
	  }

	for (ii = 0; ii < NReal; ii++)
	  {
	    n = 0;

	    n = connectionsyn ();

	    jj = 0;
	    for (jj = 0; jj < NPontosr; jj++)
	      {
		r = r0 * pow (fator, jj);	//*(1./umporrrange);
		lambdazinho = 1. - exp (-r);

		distributing ();

		lambdazinho = 1. - exp (-0.2);
		lambda[0] = lambdazinho;

		for (t = 0; t < infinity; t++)
		  {
		    iteration ();
		  }

		for (t = 0; t < infinity * 2; t++)
		  {
		    r = 1e-5;
		    lambdazinho = 1. - exp (-r);
		    lambda[0] = lambdazinho;
		    iteration ();
		    Ftotal0 = 0.0;
		    Fauxtheta1 = 0.0;
		    Fauxtheta2 = 0.0;

		    for (m = 0; m < M; m++)
		      {
			if (ESTij[m] == 2)
			  {
			    Ftotal0 = Ftotal0 + 1.0;

			    if (m < ((1.0 - iiprop) * M))
			      {
				Fauxtheta1 = Fauxtheta1 + 1;
			      }
			    else
			      {
				Fauxtheta2 = Fauxtheta2 + 1;
			      }
			    // if (m<1000)
			    {
			      fprintf (outfileRaster, "%ld\t %d \n ", t, m);
			    }
			  }
		      }

		    Ftotal0 = Ftotal0 / (1. * M);
		    Fauxtheta2 = Fauxtheta2 / (1. * M * iiprop);
		    Fauxtheta1 = Fauxtheta1 / (1. * M * (1.0 - iiprop));
		    fprintf (outfileRate, "%ld\t %g \t %g \t %g \n", t,
			     Ftotal0, Fauxtheta1, Fauxtheta2);
		  }		//end for (t)

		F0[jj][ii] = F0[jj][ii] / (1. * tmax * M);
		Ftotal[jj][ii] = Ftotal[jj][ii] / (1. * tmax * M);
		if (ii == (NReal - 1))
		  {
		    {
		      Fauxtheta[0][jj] =
			Fauxtheta[0][jj] / (1.0 * M * tmax * NReal *
					    (1.0 - iiprop));
		      Fauxtheta[1][jj] =
			Fauxtheta[1][jj] / (1.0 * M * tmax * NReal *
					    (iiprop));
		    }
		  }
	      }			//fim do for no estimulo

	  }			//fim do for no numero de real

      }				//fim do for em meansyn

    }				//fim do for em psyn
  fclose (outfileRate);
  fclose (outfileRaster);
}

/********connectionsyn***********/

int
connectionsyn (void)
{
  long int indice;

  for (m = 0; m < M; m++)
    {
      Nvizinhosauxsyn[m] = 0;
      Nvizinhossyn[m] = 0;
    }				//fim do for em m

  for (mi = 0; mi < Nsyn; mi++)	//determining the Nsyn random connections
    {
      syn[mi].ar1 = (int) (M * ran2 (&seed));


      indice = (int) (St * ran2 (&seed));

      syn[mi].ar2 = syn[mi].ar1;
      while (syn[mi].ar2 == syn[mi].ar1)
	{
	  syn[mi].ar2 = (int) (M * ran2 (&seed));
	}

      indice = (int) (St * ran2 (&seed));

      Nvizinhossyn[syn[mi].ar1]++;
      Nvizinhossyn[syn[mi].ar2]++;
    }

  for (mi = 0; mi < Nsyn; mi++)
    {
      vizinhossyn[syn[mi].ar1][Nvizinhosauxsyn[syn[mi].ar1]] = mi;
      Nvizinhosauxsyn[syn[mi].ar1]++;

      vizinhossyn[syn[mi].ar2][Nvizinhosauxsyn[syn[mi].ar2]] = mi;
      Nvizinhosauxsyn[syn[mi].ar2]++;
    }
  return 0;
}

/*******arquivo0*****/

void
arquivo0 (void)
{
  char nome_dat[80], auxi[80], radical[80];

  strcpy (radical, "rasterBimodald");
  sprintf (auxi, "%f", iiprop);
  strcat (radical, "_psynR");
  sprintf (auxi, "%f", psynR);
  strcat (radical, auxi);
  strcpy (nome_dat, radical);
  strcat (nome_dat, ".dat");
  outfileRaster = fopen (nome_dat, "w");

  strcpy (radical, "rateBimodald");
  sprintf (auxi, "%f", iiprop);
  strcat (radical, "_psynR");
  sprintf (auxi, "%f", psynR);
  strcat (radical, auxi);
  strcpy (nome_dat, radical);
  strcat (nome_dat, ".dat");
  outfileRate = fopen (nome_dat, "w");
}

/********************interactingsyn************************************/

void
interactingsyn (void)
{
  if (ESTij[m] == 1)
    {
      thetai = 0;
      for (mm = 0; mm < Nvizinhossyn[m]; mm++)
	{
	  xx = vizinhossyn[m][mm];

	  if (syn[xx].ar1 != m)
	    {
	      if (abs (ESTij[syn[xx].ar1]) == 2 && ran2 (&seed) < psyn)
		{
		  thetai++;
		  if (thetai >= thetau[m])
		    {
		      ESTij[m] = -1;
		      mm = 10000;
		    }
		}

	    }
	  else
	    {
	      if (abs (ESTij[syn[xx].ar2]) == 2 && ran2 (&seed) < psyn)
		{
		  thetai++;
		  if (thetai >= thetau[m])
		    {
		      ESTij[m] = -1;
		      mm = 10000;
		    }

		}
	    }
	}
    }
}

/*****iteration*****/
void
iteration (void)
{

  m = 0;
  for (m = 0; m < M; m++)
    {
      i = 0;
      j = 0;

      if (ESTij[m] == 1)
	{
	  if (ran2 (&seed) < lambda[0])
	    {
	      ESTij[m] = -1;
	    }
	}
      if (ESTij[m] == 1)
	{
	  interactingsyn ();

	}
      else if (ESTij[m] == 2)
	{
	  if (ran2 (&seed) < alpha)
	    {
	      ESTij[m] = -2;
	    }

	}
      else if (ESTij[m] == 3)
	{
	  if (ran2 (&seed) < beta)
	    {
	      ESTij[m] = -3;
	    }
	}
      for (i = 1; i < G; i++)
	{
	  Ni = (K + 1) * pow (K, (i - 1));

	  for (j = 0; j < Ni; j++)
	    {
	      if (ESTij[m] == 1)
		{
		  if (ran2 (&seed) < lambda[i])
		    {
		      ESTij[m] = -1;
		    }
		}
	      if (ESTij[m] == 1)
		{
		  interactingsyn ();
		}

	      else if (ESTij[m] == 2)
		{
		  if (ran2 (&seed) < alpha)
		    {
		      ESTij[m] = -2;
		    }
		}
	      else if (ESTij[m] == 3)
		{
		  if (ran2 (&seed) < beta)
		    {
		      ESTij[m] = -3;
		    }
		}

	    }			//end for j
	}			//end for em i
      if (G == 0)
	{
	}
    }				// end for m
  for (m = 0; m < M; m++)
    {

      if (ESTij[m] < 0)
	{
	  if (ESTij[m] == -1)
	    {
	      ESTij[m] = 2;

	    }

	  else if (ESTij[m] == -2)
	    ESTij[m] = 3;
	  else
	    ESTij[m] = 1;
	}
      if (ESTij[m] < 0)
	{
	  if (ESTij[m] == -1)
	    ESTij[m] = 2;

	  else if (ESTij[m] == -2)
	    ESTij[m] = 3;
	  else
	    ESTij[m] = 1;
	}
    }				// end for m
}

/*****declaring*****/

int
declaring (void)
{
  St = 0.;
  for (i = 0; i <= G; i++)
    {
      if (i == 0)
	{
	  Ni = 1;
	}
      else
	{
	  Ni = (int) (K + 1.) * pow (K, (i - 1.));
	}

      St += 1. * Ni;
    }

  Nsyn = (int) ((0.5) * meansyn * M * St);
  syn = (struct synapse *) malloc (2 * Nsyn * sizeof (struct synapse));

  if (!syn)
    {
      printf ("Allocation failure for syn\n");
      return 1;
    }
  infinity = 500;
  seed = 23048123;

  for (m = 0; m < M; m++)
    {
      for (i = 0; i <= G; i++)
	{
	  if (i == 0)
	    {
	      Ni = 1;
	    }
	  else
	    {
	      Ni = (int) ((K + 1.) * pow (K, (i - 1.)));
	    }
	}
    }
  return 0;
}				//end declaring

/*****distributing*****/

void
distributing (void)
{
  for (m = 0; m < M; m++)
    {
      ESTij[m] = 1 + (int) 3 *ran2 (&seed);

      if (m >= M * iiprop)
	{
	  thetau[m] = 1;
	}
      else
	{
	  thetau[m] = 2;
	}
    }
  lambda[0] = lambdazinho;
  Pmedio[0] = pzinho;
}				//end distributing

int
main (void)
{
  calculating ();

  free (syn);

  return 0;
}

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

float
ran2 (long *idum)
{
  int j;
  long k;
  static long idum2 = 123456789, iy = 0, iv[NTAB];
  float temp;
  if (*idum <= 0)
    {
      if (-(*idum) < 1)
	*idum = 1;
      else
	*idum = -(*idum);
      idum2 = (*idum);
      for (j = NTAB + 7; j >= 0; j--)
	{
	  k = (*idum) / IQ1;
	  *idum = IA1 * (*idum - k * IQ1) - k * IR1;
	  if (*idum < 0)
	    *idum += IM1;
	  if (j < NTAB)
	    iv[j] = *idum;
	}
      iy = iv[0];
    }
  k = (*idum) / IQ1;
  *idum = IA1 * (*idum - k * IQ1) - k * IR1;
  if (*idum < 0)
    *idum += IM1;

  k = idum2 / IQ2;
  idum2 = IA2 * (idum2 - k * IQ2) - k * IR2;
  if (idum2 < 0)
    idum2 += IM2;
  j = iy / NDIV;
  iy = iv[j] - idum2;
  iv[j] = *idum;
  if (iy < 1)
    iy += IMM1;
  if ((temp = AM * iy) > RNMX)
    return RNMX;
  else
    return temp;
}
